@@ -84,7 +84,7 @@ pub fn genvec(d: usize, n: usize, mut s1: u32, mut s2: u32) -> Vec<Vec<f64>> {
         }
         v.push(pt)
     } // fills the lot with random numbers
-    return v;
+    v
 }
 
 /// Generates a vector of n vectors, each of length d, all filled with random numbers for testing.
@@ -107,7 +107,7 @@ pub fn genvecu8(d: usize, n: usize, mut s1: u32, mut s2: u32) -> Vec<Vec<u8>> {
         }
         v.push(pt)
     } // fills the lot with random numbers
-    return v;
+    v
 }
 
 // Traits
@@ -172,6 +172,9 @@ pub trait Stats {
     /// Median and quartiles
     fn median(self) -> Result<Med>
         where Self: std::marker::Sized { bail!("median not implemented for this type")}
+    /// Zero median data, obtained by subtracting the median
+    fn zeromedian(self) -> Result<Vec<f64>>
+        where Self: std::marker::Sized { bail!("gwmeanstd not implemented for this type")}
     /// Probability density function of a sorted slice
     fn pdf(self) -> Vec<f64>;
     /// Information (entropy) in nats
@@ -239,12 +242,16 @@ pub trait Vecg<T,U> {
     fn kron(self, m:&[U]) -> Vec<f64>; 
     /// Outer product of two vectors 
     fn outer(self, m:&[U]) -> Vec<Vec<f64>>; 
-    /// Joint entropy of &[u8],&[u8] in nats 
+    /// Joint probability density function 
+    fn jointpdf(self,v:&[U]) -> Vec<f64>;     
+    /// Joint entropy of &[T],&[U] in nats (units of e)
     fn jointentropy(self, v:&[U]) -> f64;
-    /// Statistical independence measure based on joint entropy
+    /// Statistical pairwise dependence in the range [0,1] based on joint entropy
     fn dependence(self, v:&[U]) -> f64; 
     /// Pearson's correlation.  
-    fn correlation(self, v:&[U]) -> f64; 
+    fn correlation(self, v:&[U]) -> f64;
+    /// Median based correlation
+    fn mediancorr(self, v: &[U]) -> f64;
     /// Kendall Tau-B correlation. 
     fn kendalcorr(self, v:&[U]) -> f64;
     /// Spearman rho correlation.
@@ -315,14 +322,11 @@ pub trait Vecu8 {
     fn entropyu8(self) -> f64;
     /// Joint probability density function (here just co-occurence counts) 
     /// of paired values in two vectors of bytes of the same length.
-    /// Needs n^2 x 32bits of memory. Do not use for very long vectors, 
-    /// those need hashing implementation.
+    /// Needs n^2 x 32bits of memory. 
     fn jointpdfu8(self, v:&[u8]) -> Vec<Vec<u32>>;
     /// Joint entropy of &[u8],&[u8] (in nats)
     fn jointentropyu8(self, v:&[u8]) -> f64;
-    /// Dependence of two &[u8] variables, the range is [0,1],
-    /// i.e. it returns 0 iff they are statistically independent
-    /// and 1 when they are identical
+    /// Dependence in the range [0,1] of two &[u8] variables 
     fn dependenceu8(self, v:&[u8]) -> f64;
 }
 
@@ -348,6 +352,14 @@ pub trait VecVecu8 {
 /// Methods applicable to a single argument: a vector of vectors of generic end type.
 /// Operations on a set of multidimensional vectors.
 pub trait VecVec<T> {
+    /// Transpose vec of vecs like a classical array
+    fn transpose(self) -> Vec<Vec<T>>;
+    /// Joint probability density function of n matched slices of the same length
+    fn jointpdfn(self) -> Vec<f64>;
+    /// Joint entropy between a set of vectors of the same length
+    fn jointentropyn(self) -> f64;
+    /// Dependence of a set of vectors 
+    fn dependencen(self) -> f64;      
     /// Arithmetic Centre = euclidian mean of a set of points
     fn acentroid(self) -> Vec<f64>;
     /// Geometric Centroid
@@ -401,7 +413,11 @@ pub trait VecVecg<T,U> {
     fn trend(self, eps: f64, v: Vec<Vec<U>>) -> Vec<f64>;
     /// Subtract m from all points - e.g. transform to zero median form
     fn translate(self, m: &[U]) -> Vec<Vec<f64>>;
-    /// Sum of distances from arbitrary point (v) to all the points in self   
+    /// Dependencies of vector m on each vector in self
+    fn dependencies(self, m: &[U]) -> Vec<f64>;
+    /// (Median) correlations of m with each vector in self
+    fn correlations(self, m: &[U]) -> Vec<f64>;
+    /// Sum of distances from arbitrary point (v) to all the points in self      
     fn distsum(self, v: &[U]) -> f64;
     /// Individual distances from any point v (typically not in self) to all the points in self.    
     fn dists(self, v: &[U]) -> Vec<f64>;
