@@ -1,4 +1,4 @@
-use crate::{ here,wsum,wv,MStats,Med,Stats };
+use crate::{ here,wsum,wv,MStats,Med,Stats};
 use anyhow::{ensure, Result};
 // use std::ops::Sub;
 pub use indxvec::merge::{sortm,minmax};          
@@ -22,7 +22,7 @@ impl<T> Stats for &[T]
         for &component in self {
            let c = f64::from(component); 
            ensure!(c.is_normal(),
-            "{} reciprocal not allowed for zero components!\n{}\n",here!(),wv(&self)); 
+            "{} reciprocal not allowed for zero components!\n{}\n",here!(),wv(self)); 
         }     
         Ok( self.iter().map(|&x| 1.0/(f64::from(x))).collect() )     
     }
@@ -136,10 +136,7 @@ impl<T> Stats for &[T]
             })
             .sum::<f64>()
             / wsum(n);
-        Ok(MStats {
-            mean: mean,
-            std: (sx2 / wsum(n) - mean.powi(2)).sqrt(),
-        })
+        Ok(MStats { mean,std:(sx2 / wsum(n) - mean.powi(2)).sqrt()})
     }
 
     /// Harmonic mean of an f64 slice.
@@ -180,7 +177,7 @@ impl<T> Stats for &[T]
             .iter()
             .map(|&x| {
                 let fx = f64::from(x);
-                if !fx.is_normal() { panic!("{} does not accept zero valued data!",here!()) };     
+                assert!(fx.is_normal(),"{} does not accept zero valued data!",here!());     
                 let rx = 1_f64/fx;  // work with reciprocals
                 sx2 += rx * rx;
                 rx   
@@ -209,7 +206,6 @@ impl<T> Stats for &[T]
             ensure!(fx.is_normal(),"{} does not accept zero valued data!",here!());
             w += 1_f64;
             sum += w / fx;
-
         }
         Ok(wsum(n) / sum)
     }
@@ -373,9 +369,9 @@ impl<T> Stats for &[T]
         let quarter = gaps / 4;
         let threeq = 3 * gaps / 4;
         let v = sortm(self,true);     
-        let mut result: Med = Default::default();
-        result.median = if 2*mid < gaps { (f64::from(v[mid]) + f64::from(v[mid + 1])) / 2.0 }
-            else { f64::from(v[mid]) };
+        let mut result = Med { median: 
+            if 2*mid < gaps { (f64::from(v[mid]) + f64::from(v[mid + 1])) / 2.0 }
+            else { f64::from(v[mid]) }, ..Default::default() };
         match gaps % 4 {
         0 => {
             result.lquartile = f64::from(v[quarter]);
@@ -396,34 +392,37 @@ impl<T> Stats for &[T]
         _ => { }  
         }
         Ok(result)       
-    } 
+    }
 
-    /// Probability density function of a sorted slice with repeats
+    /// Zero median data produced by subtracting the median.
+    /// Analogous to zero mean data when subtracting the mean.
+    fn zeromedian(self) -> Result<Vec<f64>> {
+        let Med{median,..} = self.median() // ignore quartile fields
+            .unwrap_or_else(|_|panic!("{} failed to obtain median",here!()));
+        Ok(self.iter().map(|&s| f64::from(s)-median).collect())
+    }
+
+    /// Probability density function of a sorted slice with repeats. 
+    /// Repeats are counted and removed
     fn pdf(self) -> Vec<f64> {     
-        let n = self.len();   
-        let mut res:Vec<f64> = vec![1.0;n];  
-        let mut rcount = 1_f64; // running count
-        let mut lastval = self[0]; // first item
-
-        for i in 1..n { // first pass to count same values 
-            if self[i] > lastval { lastval = self[i]; rcount = 1.0 } // new value                 
-            else { rcount += 1.0 } // same value
-            res[i] = rcount            
-        }
-        let nf = n as f64;
-        for i in (0..n).rev() {  // second reverse pass to propagate maxima
-            if self[i] < lastval { lastval = self[i]; rcount = res[i] }; // new value                
-            res[i] = rcount/nf // propagate maximum count from previous item           
-        } 
-        // let ressum = res.iter().sum::<f64>();
-        // eprintln!("Sum of probs: {}",ressum); 
+        let nf = self.len() as f64;   
+        let mut res:Vec<f64> = Vec::new();  
+        let mut count = 1_usize; // running count
+        let mut lastval = self[0];
+        self.iter().skip(1).for_each(|&s| (
+            if s > lastval { // new value encountered
+                res.push((count as f64)/nf); // save previous probability
+                lastval = s; // new value
+                count = 1_usize; // reset counter
+            } else { count += 1; }));
+        res.push((count as f64)/nf);  // flush the rest!
         res
     } 
 
     /// Information (entropy) (in nats)
     fn entropy(self) -> f64 {
         let pdfv = sortm(self,true).pdf();      
-        pdfv.iter().map( |&x| -x*(x.ln()) ).sum()                 
+        pdfv.iter().map(|&x| -x*(x.ln()) ).sum()                 
     }
 
     /// (Auto)correlation coefficient of pairs of successive values of (time series) f64 variable.
@@ -438,15 +437,15 @@ impl<T> Stats for &[T]
         let n = self.len();
         if n < 2 { panic!("{} vector is too short",here!()) }
         let mut x = f64::from(self[0]);    
-        for i in 1..n {
-            let y = f64::from(self[i]);
+        self.iter().skip(1).for_each(|&si| {
+            let y = f64::from(si);
             sx += x;
             sy += y;
             sxy += x * y;
             sx2 += x * x;
             sy2 += y * y;
             x = y
-        }        
+        });        
         let nf = n as f64;
         (sxy - sx / nf * sy) / ((sx2 - sx / nf * sx) * (sy2 - sy / nf * sy)).sqrt()
     }
@@ -456,23 +455,25 @@ impl<T> Stats for &[T]
         let range = f64::from(mm.max)-f64::from(mm.min);
         self.iter().map(|&x|(f64::from(x)-f64::from(mm.min))/range).collect()        
     }
+
     /// Reconstructs the full symmetric square matrix from its lower diagonal compact form,
     /// as produced by covar, covone, wcovar
     fn symmatrix(self) -> Vec<Vec<f64>> {
-        // solve quadratic equation to find the dimension of the square matrix
-        let n = (((8*self.len()+1) as f64).sqrt() as usize - 1)/2;
-        let mut mat = vec![vec![0_f64;n];n]; // create the square matrix 
-        let mut selfindex = 0;
-        for row in 0..n {     
-            for column in 0..row { // excludes the diagonal 
-                let this = f64::from(self[selfindex]); 
-                mat[row][column] = this; // just copy the value into the lower triangle
-                mat[column][row] = this; // and into transposed upper position 
-                selfindex += 1 // move to the next input value
-            } // this row of lower triangle finished
-            mat[row][row] = f64::from(self[selfindex]);  // now the diagonal element, no transpose
-            selfindex += 1 // move to the next input value
+
+         fn trseqtosubs(s:usize) -> (usize,usize) { 
+            // solution of quadratic equation to find the dimension of the full square matrix
+            let row = ((((8*s+1) as f64).sqrt() - 1.)/2.) as usize; // cast truncates, like .floor()
+            let column = s - row*(row+1)/2; // subtracting previous triangular number
+            (row,column)
         }
+
+        let (n,_) = trseqtosubs(self.len());
+        let mut mat = vec![vec![0_f64;n];n]; // create the square matrix 
+        self.iter().enumerate().for_each(|(i,&s)| {
+            let (row,column) = trseqtosubs(i);
+            if row < column { mat[column][row] = f64::from(s) as f64; }; // symmetrical reflection
+            // also set values in lower triangular region, including the diagonal
+            mat[row][column] = f64::from(s); } ); 
         mat
     }    
  
