@@ -1,11 +1,62 @@
 use std::iter::FromIterator;
 
 use crate::{Med, MStats, MinMax, MutVecg, MutVecf64, Stats, Vecg, Vecf64, VecVec};
-pub use indxvec::{merge::*,Indices};
+pub use indxvec::{here,merge::*,Indices};
 
 impl<T> VecVec<T> for &[Vec<T>] where T: Copy+PartialOrd+std::fmt::Display,
     f64: From<T> {
 
+    /// Transpose vec of vecs like a matrix
+    fn transpose(self) -> Vec<Vec<T>> {
+        let n = self.len();
+        let d = self[0].len();
+        let mut transp:Vec<Vec<T>> = Vec::with_capacity(d);
+        for i in 0..d { 
+            let mut column = Vec::with_capacity(n);
+            for v in self {
+                column.push(v[i]);
+            }
+            transp.push(column); // column becomes row
+        }
+        transp   
+    }
+
+    /// Joint probability density function of n matched slices of the same length
+    fn jointpdfn(self) -> Vec<f64> {     
+        let d = self[0].len(); // their common dimensionality (length)
+        for v in self.iter().skip(1) {
+            if v.len() != d { panic!("{} all vectors must be of equal length!",here!()) }; 
+        }
+        let mut res:Vec<f64> = Vec::with_capacity(d);
+        let mut tuples = self.transpose();
+        let df = d as f64; // for turning counts to probabilities
+        // lexical sort to group together occurrences of identical tuples
+        tuples.sort_unstable_by(|a, b| a.partial_cmp(b).unwrap()); 
+        let mut count = 1_usize; // running count
+        let mut lastindex = 0; // initial index of the last unique tuple
+        tuples.iter().enumerate().skip(1).for_each(|(i,ti)|
+            if ti > &tuples[lastindex] { // new tuple ti (Vec<T>) encountered
+                res.push((count as f64)/df); // save frequency count as probability
+                lastindex = i; // current index becomes the new one
+                count = 1_usize; // reset counter
+            } else { count += 1; } );        
+        res.push((count as f64)/df);  // flush the rest!
+        res
+    } 
+
+    /// Joint entropy of vectors of the same length
+    fn jointentropyn(self) -> f64 {
+        let jpdf = self.jointpdfn();
+        jpdf.iter().map(|&x| -x*(x.ln()) ).sum() 
+    }
+
+    /// Dependence (component wise) of a set of vectors.
+    /// i.e. `dependencen` returns 0 iff they are statistically independent
+    fn dependencen(self) -> f64 { 
+        let individuales:f64 = self.iter().map(|v| v.entropy()).sum();
+        individuales - self.jointentropyn() 
+    } 
+
     /// acentroid = simple multidimensional arithmetic mean
     /// # Example
     /// ```
@@ -16,7 +67,7 @@ impl<T> VecVec<T> for &[Vec<T>] where T: Copy+PartialOrd+std::fmt::Display,
     /// ```
     fn acentroid(self) -> Vec<f64> {
         let mut centre = vec![0_f64; self[0].len()];
-        for v in self { centre.mutvadd(&v) }
+        for v in self { centre.mutvadd(v) }
         centre.mutsmultf64(1.0 / (self.len() as f64));
         centre
     }
@@ -30,15 +81,15 @@ impl<T> VecVec<T> for &[Vec<T>] where T: Copy+PartialOrd+std::fmt::Display,
     /// assert_eq!(dist,4.897594485332543_f64);
     /// ```
     fn gcentroid(self) -> Vec<f64> {
-        let n = self.len() as f64; // number of points
+        let nf = self.len() as f64; // number of points
         let d = self[0].len(); // dimensions
         let mut centre = vec![0_f64; d];
         let mut lnvec = vec![0_f64; d];
         for v in self { 
-            for i in 0..d { lnvec[i] = f64::from(v[i]).ln() }
+            v.iter().zip(&mut lnvec).for_each(|(&vi,lni)| *lni = f64::from(vi).ln() );
             centre.mutvaddf64(&lnvec)
         }
-        centre.iter().map(|comp| (comp/n).exp()).collect()        
+        centre.iter().map(|comp| (comp/nf).exp()).collect()        
     }
 
     /// hcentroid =  multidimensional harmonic mean
@@ -52,9 +103,7 @@ impl<T> VecVec<T> for &[Vec<T>] where T: Copy+PartialOrd+std::fmt::Display,
     /// ```
     fn hcentroid(self) -> Vec<f64> {
         let mut centre = vec![0_f64; self[0].len()]; 
-        for v in self {
-            centre.mutvaddf64(&v.vinverse().unwrap())
-        }
+        for v in self { centre.mutvaddf64(&v.vinverse().unwrap()) }
         centre.smultf64(1.0/(self.len() as f64)).vinverse().unwrap()       
     }
 
@@ -62,32 +111,22 @@ impl<T> VecVec<T> for &[Vec<T>] where T: Copy+PartialOrd+std::fmt::Display,
     fn distsums(self) -> Vec<f64> {
         let n = self.len();
         let mut dists = vec![0_f64; n]; // distances accumulator for all points
-                                        // examine all unique pairings (lower triangular part of symmetric flat matrix)
-        for i in 1..n {
-            let thisp = &self[i];
-            for j in 0..i {
-                let thatp = &self[j];
-                let d = thisp.vdist(&thatp); // calculate each distance relation just once
+        // examine all unique pairings (lower triangular part of symmetric flat matrix)
+        self.iter().enumerate().for_each(|(i,thisp)| 
+            self.iter().take(i).enumerate().for_each(|(j,thatp)| {
+                let d = thisp.vdist(thatp); // calculate each distance relation just once
                 dists[i] += d;
-                dists[j] += d; // but add it to both points
-            }
-        }     
+                dists[j] += d; // but add it to both points' sums
+            }));
         dists
     } 
 
     /// The sum of distances from one member point, given by its `indx`, to all the other points in self.
     /// For all the points, use more efficient `distsums`.    
-    fn distsuminset(self, indx: usize) -> f64 {
-        let n = self.len();
-        let mut sum = 0_f64;
+    fn distsuminset(self, indx: usize) -> f64 { 
         let thisp = &self[indx];
-        for i in 0..n {
-            if i == indx {
-                continue;
-            };
-            sum += self[i].vdist(&thisp)
-        }
-        sum
+        self.iter().enumerate().map(|(i,thatp)|
+            if i == indx { 0.0 } else {thisp.vdist(thatp)}).sum()
     } 
 
     /// Medoid and Outlier (Medout)
@@ -108,7 +147,7 @@ impl<T> VecVec<T> for &[Vec<T>] where T: Copy+PartialOrd+std::fmt::Display,
     }
 
     /// Finds approximate vectors from each member point towards the geometric median.
-    /// Twice as fast as doing them individually, using symmetry.
+    /// Twice as fast using symmetry, as doing them individually.
     fn eccentricities(self) -> Vec<Vec<f64>> {
         let n = self.len();
         // allocate vectors for the results
@@ -120,14 +159,14 @@ impl<T> VecVec<T> for &[Vec<T>] where T: Copy+PartialOrd+std::fmt::Display,
             let thisp = &self[i];
             for j in 0..i { 
                 // calculate each unit vector between any pair of points just once
-                let dvmag = self[j].vdist(&thisp);             
+                let dvmag = self[j].vdist(thisp);             
                 if !dvmag.is_normal() { continue }
                 let rec = 1.0_f64/dvmag;
                 eccs[i].mutvaddf64(&self[j].smultf64(rec));
                 recips[i] += rec;
                 // mind the vector's opposite orientations w.r.t. to the two points!
                 eccs[j].mutvsubf64(&self[j].smultf64(rec)); 
-                recips[j] += rec;
+                recips[j] += rec; // but scalar distances are the same
             }
         }
         for i in 0..n { 
@@ -143,7 +182,7 @@ impl<T> VecVec<T> for &[Vec<T>] where T: Copy+PartialOrd+std::fmt::Display,
         let mut eccs = Vec::with_capacity(self.len()); // Vectors for the results
         let gm:Vec<f64> = self.gmedian(eps);
         for v in self {
-            eccs.push(gm.vsub(&v))
+            eccs.push(gm.vsub(v))
         }
         eccs
     }
@@ -173,7 +212,7 @@ impl<T> VecVec<T> for &[Vec<T>] where T: Copy+PartialOrd+std::fmt::Display,
     /// These are new robust measures of a cloud of multidimensional points (or multivariate sample).  
     fn eccinfo(self, eps: f64) -> (MStats, Med, MinMax<f64>) where Vec<f64>:FromIterator<f64> {
         let gm:Vec<f64> = self.gmedian(eps);
-        let eccs:Vec<f64> = self.iter().map(|v| gm.vdist(&v)).collect();
+        let eccs:Vec<f64> = self.iter().map(|v| gm.vdist(v)).collect();
         (eccs.ameanstd().unwrap(),eccs.median().unwrap(),minmax(&eccs))
     }
      
@@ -182,7 +221,7 @@ impl<T> VecVec<T> for &[Vec<T>] where T: Copy+PartialOrd+std::fmt::Display,
     fn sortedeccs(self, ascending:bool, gm:&[f64]) -> Vec<f64> { 
         let mut eccs = Vec::with_capacity(self.len()); 
         // collect raw ecentricities magnitudes
-        for v in self { eccs.push(v.vdistf64(&gm)) }
+        for v in self { eccs.push(v.vdistf64(gm)) }
         sortm(&eccs,ascending)
     }    
 
@@ -200,7 +239,7 @@ impl<T> VecVec<T> for &[Vec<T>] where T: Copy+PartialOrd+std::fmt::Display,
     /// ```
     fn emedoid(self, eps: f64) -> MinMax<f64> where Vec<f64>:FromIterator<f64> {
     let gm:Vec<f64> = self.gmedian(eps);
-    let eccs:Vec<f64> = self.iter().map(|v| gm.vdist(&v)).collect();
+    let eccs:Vec<f64> = self.iter().map(|v| gm.vdist(v)).collect();
     minmax(&eccs)
 } 
 
@@ -233,19 +272,19 @@ impl<T> VecVec<T> for &[Vec<T>] where T: Copy+PartialOrd+std::fmt::Display,
     /// Next approximate gm computed from a member point  
     /// specified by its index `indx` to self. 
     fn nxmember(self, indx: usize) -> Vec<f64> {
-        let n = self.len();
         let mut vsum = vec![0_f64; self[0].len()];
         let p = &self[indx];
         let mut recip = 0_f64;
-        for i in 0..n {
-            if i == indx { continue  }; // exclude this point
-            let pi = &self[i];
-            let mag = p.vdist(pi);
-            if !mag.is_normal() { continue } // too close to this one
-            let rec = 1.0_f64/mag;
-            vsum.mutvaddf64(&pi.smultf64(rec)); // add vector
-            recip += rec // add separately the reciprocals
-        }
+        self.iter().enumerate().for_each(|(i,thatp)| {
+            if i != indx {  // not point p
+                let mag = p.vdist(thatp);
+                if mag.is_normal() {  // not too close to p
+                    let rec = 1.0_f64/mag;
+                    vsum.mutvaddf64(&thatp.smultf64(rec)); // add vector
+                    recip += rec // add separately the reciprocals
+                }
+            }
+        });
         vsum.mutsmultf64(1.0/recip);
         vsum 
     }
