@@ -5,27 +5,29 @@
 use devtimer::DevTime;
 use anyhow::Result;
 use indxvec::{Indices,merge::*};
-use rstats::{Stats,MutVecg,Vecg,VecVec,VecVecg,Vecu8};
+use rstats::{Stats,MutVecg,Vecg,VecVec,VecVecg,Vecu8, Vecf64};
 use rstats::{wv,wi,printvv,genvec,genvecu8,i64tof64,tof64};
 
 pub const EPS:f64 = 1e-10;
 #[test]
 fn u8() -> Result<()> {
    let v1 = vec![1_u8,2,2,3,3,3,4,4,4,4,5,5,5,5,5,6,6,6,6,6,6]; 
-   println!("\n{}",wv(&v1));
-   println!("Entropy: {}",wi(&v1.entropy()));
+   println!("\nv1: {}",wv(&v1));
    let v2 = vec![1_u8,2,2,3,3,3,4,4,4,4,3,3,3,3,3,3,2,2,2,2,2]; 
-   println!("{}",wv(&v2)); 
-   println!("Entropy: {}",wi(&v2.entropy()));
-   println!("Entropy: {}",wi(&v2.entropyu8()));
-   println!("|v2-v1|: {}",wi(&v2.vdistu8(&v1)));
-   println!("Cityblockd: {}",wi(&v2.cityblockd(&v1)));   
-   println!("Cityblockd: {}",wi(&v2.cityblockdu8(&v1))); 
-   println!("Joint E: {}",wi(&v1.jointentropy(&v2)));
-   println!("Joint E: {}",wi(&v1.jointentropyu8(&v2)));
-   println!("Dependence: {}",wi(&v1.dependence(&v2)));
-   println!("Dependence: {}",wi(&v1.dependenceu8(&v2)));
-   println!("SpearCorr:  {}",wi(&v1.spearmancorr(&v2)));
+   println!("v2: {}",wv(&v2)); 
+   println!("Lexical order v1<v2: {}", wi(&(v1<v2)));
+   println!("Entropy 1:\t{}",wi(&v1.entropy()));
+   println!("Entropy 2 gen:\t{}",wi(&v2.entropy())); // generic
+   println!("Entropy 2 u8:\t{}",wi(&v2.entropyu8())); // u8
+   println!("Euclid's dist:\t{}",wi(&v2.vdistu8(&v1)));
+   println!("Cityblock dist:\t{}",wi(&v2.cityblockd(&v1)));
+   println!("Joint Entropy gen: {}",wi(&v1.jointentropy(&v2)));   
+   println!("Joint Entropy u8:  {}",wi(&v1.jointentropyu8(&v2)));
+   println!("Generic dep.:\t{}",wi(&v1.dependence(&v2))); // generic
+   println!("Dependence u8:\t{}",wi(&v1.dependenceu8(&v2))); // u8
+   println!("Cos:\t\t{}",wi(&v1.cosineu8(&v2)));
+   println!("Pearson Correlation:  {}",wi(&v1.correlation(&v2)));  
+   println!("Spearman Correlation: {}",wi(&v1.spearmancorr(&v2)));
    let d = 5_usize;
    let n = 7_usize;
    println!("Testing on a random set of {} points in {} d space:",wi(&n),wi(&d));
@@ -104,28 +106,35 @@ fn genericstats() -> Result<()> {
 }
 #[test]
 fn vecg() -> Result<()> { 
-   let v1 = vec![1_f64,2.,3.,4.,5.,6.,7.,8.,9.,10.,11.,12.,13.,14.,15.];
+   let v1 = vec![1_f64,2.,3.,4.,5.,6.,7.,8.,9.,10.,10.,10.,13.,14.,15.];
    println!("v1: {}",wv(&v1));
-   let v2 = vec![1_f64,14.,2.,13.,3.,12.,4.,11.,5.,10.,6.,9.,7.,8.,15.];
+   let v2 = vec![1_f64,14.,2.,13.,3.,12.,4.,11.,5.,10.,6.,9.,7.,1.,15.];
    println!("v2: {}",wv(&v2)); 
+   println!("Lexical order v1<v2:\t{}", wi(&(v1<v2)));
    println!("Pearson's Correlation:\t{}",wi(&v1.correlation(&v2))); 
+   println!("Median Correlation:\t{}",wi(&v1.mediancorr(&v2))); 
    println!("Kendall's Correlation:\t{}",wi(&v1.kendalcorr(&v2)));  
    println!("Spearman's Correlation:\t{}",wi(&v1.spearmancorr(&v2)));  
-   println!("Cosine:\t\t\t{}",wi(&v1.cosine(&v2))); 
-   println!("Cosine of ranks:\t{}",
-      wi(&rank(&v1,true).indx_to_f64().cosine(&rank(&v2,true).indx_to_f64())));        
-   println!("Euclidian distance:\t{}",wi(&v1.vdist(&v2)));
-   println!("Difference magnitude:\t{}",wi(&v1.vsub(&v2).vmag()));   
+   println!("Euclidian distance:\t{}",wi(&v1.vdistf64(&v2)));
+   println!("Cityblock distance:\t{}",wi(&v1.cityblockd(&v2)));   
    println!("Vector difference: {}",wv(&v1.vsub(&v2))); 
    println!("Vector sum: {}",wv(&v1.vadd(&v2)));  
    println!("Scalar product:\t\t{}",wi(&v1.dotp(&v2)));
    println!("Parallelogram area:\t{}",wi(&v1.varea(&v2)));
    println!("Arc area:\t\t{}",wi(&v1.varc(&v2))); 
+   println!("Entropy v1:\t\t{}",wi(&v1.entropy()));
+   println!("Entropy v2:\t\t{}",wi(&v2.entropy()));
+   println!("Joint Entropy:\t\t{}",wi(&v1.jointentropy(&v2)));
    println!("Dependence:\t\t{}",wi(&v1.dependence(&v2)));
-   println!("Similarity:\t\t{}",wi(&v1.vsim(&v2)));
-   println!("Dissimilarity:\t\t{}",wi(&v1.vdisim(&v2))); 
+   println!("Cosine:\t\t\t{}",wi(&v1.cosine(&v2))); 
+   println!("Cosine of ranks:\t{}",
+      wi(&rank(&v1,true).indx_to_f64().cosine(&rank(&v2,true).indx_to_f64())));        
+   println!("Cos Similarity [0,1]:\t{}",wi(&v1.vsim(&v2)));
+   println!("Cos Dissimilarity:\t{}",wi(&v1.vdisim(&v2))); 
    println!("[1,2,3].kron(&[4,5]):\t{}", wv(&[1,2,3].kron(&[4,5])));
-   println!("[1,2,3].outer(&[4,5,6,7]): "); printvv([1,2,3].outer(&[4,5,6,7]));
+   let outerp = [1,2,3].outer(&[4,5,6,7]);
+   println!("[1,2,3].outer(&[4,5,6,7]): "); printvv(outerp);
+   // println!("Transposed: "); printvv([1,2,3].outer(&[4,5,6,7]).transpose()); 
    Ok(())
 }
 
@@ -142,14 +151,18 @@ fn trend() -> Result<()> {
 
 #[test]
 fn vecvec() -> Result<()> { 
-   let d = 13_usize;
-   let n = 108_usize;
+   let d = 55_usize;
+   let n = 13_usize;
    println!("testing on a random set of {} points in {} dimensional space",wi(&n),wi(&d));
    let mut weights = Vec::new();
    for i in 1..n+1 { weights.push(i as f64) }; // create test weights data
    let pt = genvecu8(d,n,5,17); // random u8 test data
-   let (eccstd,eccmed,eccecc) = pt.eccinfo(EPS);
- 
+   println!("Joint entropy: {}",wi(&pt.jointentropyn()) );  
+   println!("Statistical component wise dependence: {}",wi(&pt.dependencen()) ); 
+   let outcomes = &genvecu8(d,1,7,19)[0];  
+   println!("Dependencies of outcomes: {}",wv(&pt.dependencies(outcomes))); 
+   println!("Correlations with outcomes: {}",wv(&pt.correlations(outcomes)));
+   let (eccstd,eccmed,eccecc) = pt.eccinfo(EPS); 
    // let me = pt.emedoid(EPS);
    let medoid = &pt[eccecc.minindex];
    let outlier = &pt[eccecc.maxindex];
@@ -170,7 +183,7 @@ fn vecvec() -> Result<()> {
    println!("GCentroid's total distances:\t{}",wi(&pt.distsum(&gcentroid)));
    println!("ACentroid's total distances:\t{}",wi(&pt.distsum(&acentroid)));
    println!("Median's total distances:\t{}",wi(&pt.distsum(&median)));  
-   println!("Outlier's distance to Medoid:\t{}",wi(&outlier.vdist(&medoid)));      
+   println!("Outlier's distance to Medoid:\t{}",wi(&outlier.vdist(medoid)));      
    println!("Outlier's radius (from Median):\t{}",wi(&outlier.vdist(&median)));  
    println!("Medoid's radius (from Median):\t{}",wi(&medoid.vdist(&median)));
 
